# ------------------------------------------
# BPM Converter v.1.05
# Author : VIENTE
# ------------------------------------------

### Define variables and functions ####################################################################

numItems = RPR_CountSelectedMediaItems(0)
selItem = RPR_GetSelectedMediaItem(0, 0)

def msg(m):					# Define function: console output alias for debugging
	RPR_ShowConsoleMsg(str(m) + '\n')



def getMusicalLenght(selItem):
	
	# Get selected item's start and end
	itemStart = RPR_GetMediaItemInfo_Value(selItem, "D_POSITION")
	itemEnd = itemStart + RPR_GetMediaItemInfo_Value(selItem, "D_LENGTH")

	# Check if there are any tempo changes during item's length (NextChangeTime will output -1 if there are no more tempo markers after the requested position)
	if RPR_TimeMap2_GetNextChangeTime(0, itemStart) > itemEnd or RPR_TimeMap2_GetNextChangeTime(0, itemStart) == -1: 
		
		# There are no tempo markers over selected item so we continue
		timeSig = RPR_TimeMap_GetTimeSigAtTime(0, itemStart, 0, 0, 0)

		# This is needed because then we don't have to worry about linear/square difference of tempo points
		startBPM = timeSig[4]
		endBPM = RPR_TimeMap_GetTimeSigAtTime(0, itemStart, 0, 0, 0) [4] # we don't use TimeMap_GetDividedBpmAtTime because note in the API says: get the effective BPM at the time (seconds) position (i.e. 2x in /8 signatures) 
		
		# This is the formula
		global musicalLenght
		musicalLenght = ((startBPM + endBPM) * timeSig[3] * (itemEnd - itemStart)) / (480 * timeSig[2])
	
	else:
		# if there are existing tempo markers during item's length, you have to get them and calculate everything separately
		RPR_Main_OnCommand(65535, 0)

def getTempo():

	global tempo
	global bpm

	timeSel = RPR_GetMediaItemInfo_Value(selItem, "D_LENGTH")
	rawTempo = 4*60/timeSel
	tempo = round(rawTempo, 3)

	if musicalLenght < 1 or musicalLenght == 1:

		bpm = tempo

	elif musicalLenght > 1 and musicalLenght < 3:

		bpm = tempo * 2

	elif musicalLenght > 3 and musicalLenght < 5:

		bpm = tempo * 4

	elif musicalLenght > 5 and musicalLenght < 7:

		bpm = tempo * 6

	elif musicalLenght > 7 and musicalLenght < 9:

		bpm = tempo * 8

	elif musicalLenght > 9 and musicalLenght < 11:

		bpm = tempo * 10

	elif musicalLenght > 11 and musicalLenght < 13:

		bpm = tempo * 12

	elif musicalLenght > 13 and musicalLenght < 15:

		bpm = tempo * 14

	elif musicalLenght > 15 and musicalLenght < 17:

		bpm = tempo * 16

	else:

		bpm = 1

	#RPR_ShowConsoleMsg(str(bpm) + "\n")

def setTempo():

	master_tempo = int(RPR_Master_GetTempo())
	names = "Original Tempo (BPM),Target Tempo (BPM)"
	defaults = "%d,%d" % (bpm, master_tempo)
	user_tempo = RPR_GetUserInputs('BPM Converter', 2, names, defaults, 1024);

	if user_tempo[0]:

		tempo_values = user_tempo[4].split(',')
		answer1 = int(tempo_values[0])
		answer2 = int(tempo_values[1])

		if answer1 > 20 and answer1 < 299 and answer2 > 20 and answer2 < 299:

			if answer2 > answer1:

				bpm_rate = (answer2 / answer1) -1
				item = RPR_GetSelectedMediaItem(0, 0)
				take = RPR_GetActiveTake(item)
				currentrate = RPR_GetMediaItemTakeInfo_Value(take, "D_PLAYRATE")
				RPR_SetMediaItemTakeInfo_Value(take, "D_PLAYRATE", currentrate + bpm_rate)

			elif answer2 < answer1:

				bpm_rate = (answer2 / answer1)
				bpm_calc = (1 - bpm_rate)
				item = RPR_GetSelectedMediaItem(0, 0)
				take = RPR_GetActiveTake(item)
				currentrate = RPR_GetMediaItemTakeInfo_Value(take, "D_PLAYRATE")
				RPR_SetMediaItemTakeInfo_Value(take, "D_PLAYRATE", currentrate - bpm_calc)

		else:
			RPR_ShowMessageBox("Incorrect tempo!", "Error", 0) # 0=OK,2=OKCANCEL,2=ABORTRETRYIGNORE,3=YESNOCANCEL,4=YESNO,5=RETRYCANCEL : ret 1=OK,2=CANCEL,3=ABORT,4=RETRY,5=IGNORE,6=YES,7=NO		

### Calling actions & functions #######################################################################

RPR_Undo_BeginBlock()

if numItems == 1:

	getMusicalLenght(selItem)
	getTempo()
	setTempo()
	RPR_Main_OnCommand(40612,0) # Fix item length
	RPR_UpdateArrange()

else:
	RPR_ShowMessageBox("Please select only one item...", "Error", 0) # 0=OK,2=OKCANCEL,2=ABORTRETRYIGNORE,3=YESNOCANCEL,4=YESNO,5=RETRYCANCEL : ret 1=OK,2=CANCEL,3=ABORT,4=RETRY,5=IGNORE,6=YES,7=NO

RPR_Undo_EndBlock("Convert BPM of selected item",0)